﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Netron.GraphLib;
using Netron.GraphLib.Interfaces;

namespace FlowChart
{
    public class SequenceShape : Shape, IConnectable
    {
        private TextBox m_tb;
        private Connector m_leftConnector;
        private Connector m_rightConnector;

        protected override void InitEntity()
        {
            base.InitEntity();

            m_leftConnector = new Connector(this,"Left",true);
            Connectors.Add(m_leftConnector);

            m_rightConnector = new Connector(this, "Right", true);
            Connectors.Add(m_rightConnector);

            Text = "顺序图形";
            Font = new Font("宋体", 10);

            Pen = new Pen(Color.FromArgb(167, 58, 95));
            ShapeColor = Color.FromArgb(255, 253, 205);

            Rectangle = new RectangleF(100, 100, 100, 50);

            OnMouseDown += SequenceShape_OnMouseDown;

             

        }

        public override PointF ConnectionPoint(Connector c)
        {
            if (c == m_leftConnector)
            {
                return new PointF(Rectangle.Left, Rectangle.Top + Rectangle.Height / 2);
            }

            if (c == m_rightConnector)
            {
                return new PointF(Rectangle.Right, Rectangle.Top + Rectangle.Height / 2);
            }

            return new PointF(0, 0);
        }

        public override void Paint(Graphics g)
        {
            base.Paint(g);

            g.FillRectangle(new SolidBrush(ShapeColor), Rectangle);
            g.DrawRectangle(Pen, System.Drawing.Rectangle.Round(Rectangle));

            if (!string.IsNullOrEmpty(Text))
                g.DrawString(Text, this.Font, this.TextBrush, System.Drawing.RectangleF.Inflate(Rectangle, 0, -2));

        }


        private void SequenceShape_OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (e.Clicks == 2 && e.Button == MouseButtons.Left)
            {
                if (m_tb == null)
                {
                    m_tb = new TextBox();
                }

                m_tb.Location = System.Drawing.Point.Round(Rectangle.Location);
                m_tb.Width = (int) Rectangle.Width;
                m_tb.Height = (int)Rectangle.Height;
                m_tb.BackColor = ShapeColor;
                m_tb.Multiline = true;
                m_tb.Text = Text;
                m_tb.SelectionLength = Text.Length;

                m_tb.LostFocus += T_tb_LostFocus;
                (Site as Control).Controls.Add(m_tb);
                m_tb.Show();
                m_tb.Focus();
                m_tb.ScrollToCaret();
            }
        }

        private void T_tb_LostFocus(object sender, EventArgs e)
        {
            Text = m_tb.Text;
            m_tb.Hide();
            (Site as Control).Controls.Remove(m_tb);
        }

        public Connection CreateConnection(Connector connector)
        {
            return new FlowChartConnection();
        }
    }
}